/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};
gc.services = gc.services || {};

var cntNACK = 0;
var chartAutoMode = 0;
var chartTrigDur = 0;
var jdrift_en = false;
var pTMP = 0;

const U2A =
{
    I2C_Control: 1, // 0x01
    I2C_Write: 2, // 0x02
    I2C_Read: 3, // 0x03
    I2CRead_WithAddress: 4 // 0x04
}

const convT = function (bytes) {
    return -45 + 175 * (bytes / (2 ** 16 - 1));
}
const convR = function (bytes) {
    return 100 * (bytes / (2 ** 16 - 1));
}
// crc function takes array input
// ex: crcHDC3([0xAB, 0xCD]) = 0x6F
const crcHDC3 = function (msg) {
    let crc = 0xFF;
    for (i = 0; i < msg.length; i++) {
        crc ^= msg[i];
        for (bit = 0; bit < 8; bit++) {
            if (crc & 0x80)
                crc = (crc << 1) ^ 0x31;
            else
                crc = (crc << 1);
        }
    }
    return crc & 0xFF;
};

const hdc3 = {
    mode: {
        trig: {
            MSB: 0x24,
            duration: [0x00, 0x0B, 0x16, 0xFF]
        },
        a2s: {
            MSB: 0x20,
            duration: [0x32, 0x24, 0x2F, 0xFF]
        },
        a1hz: {
            MSB: 0x21,
            duration: [0x30, 0x26, 0x2D, 0xFF]
        },
        a2hz: {
            MSB: 0x22,
            duration: [0x36, 0x20, 0x2B, 0xFF]
        },
        a4hz: {
            MSB: 0x23,
            duration: [0x34, 0x22, 0x29, 0xFF]
        },
        a10hz: {
            MSB: 0x27,
            duration: [0x37, 0x21, 0x2A, 0xFF]
        }
    },
    cfgAlert: {
        MSB: 0x61,
        trgLo: 0x00,
        trgHi: 0x1D,
        clrLo: 0x0B,
        clrHi: 0x16
    },
    vrfyAlert: {
        MSB: 0xE1,
        trgLo: 0x02,
        trgHi: 0x1F,
        clrLo: 0x09,
        clrHi: 0x14
    }
}

const HDC3Write = function (msg) {
    const u2a = document.querySelector('ti-model-register').getCodec();
    const HDC3Addr = templateObj.$.ti_widget_droplist_i2c_addr.selectedValue;
    const Cmd_I2C_Write = 2;
    u2a.readResponse(u2a.sendCommandPacket(Cmd_I2C_Write, [].concat(0, HDC3Addr, msg.length, msg)));
}

const HDC3Read = function (bytes, callback) {
    const u2a = document.querySelector('ti-model-register').getCodec();
    const HDC3Addr = templateObj.$.ti_widget_droplist_i2c_addr.selectedValue;
    const Cmd_I2C_Read = 3;
    const commandSent = u2a.sendCommandPacket(Cmd_I2C_Read, [0, HDC3Addr, bytes, 0]);
    u2a.readResponse(commandSent).then(function (responsePacket) {
        const payloadLength = responsePacket[2];
        const payload = responsePacket.slice(8, 8 + payloadLength);
        callback(payload);
    });
}

const myPlotly = {};
myPlotly.update = (y1, y2) => {
    if (templateObj) {
        const myPlot = templateObj.$.ti_widget_plot;
        if (typeof myPlot.data[1] == 'undefined') {
            const ti_red = '#CC0000';
            const ti_blue = '#115566';
            myPlot.data.push({ x: Array(), y: Array() });
            myPlot.data[0].name = 'RH %';
            myPlot.data[1].name = '°C';
            myPlot.data[1].yaxis = 'y2';
            myPlot.data[0].line = { color: ti_red };
            myPlot.data[1].line = { color: ti_blue };
            myPlot.layout.yaxis = {
                title: 'RH %',
                range: [0, 100],
                titlefont: { color: ti_red },
                tickfont: { color: ti_red }
            };
            myPlot.layout.yaxis2 = {
                title: '°C',
                range: [0, 100],
                titlefont: { color: ti_blue },
                tickfont: { color: ti_blue },
                overlaying: 'y',
                side: 'right'
            };
        };
        if (myPlot.data[0].x.length > 99) {
            myPlot.data[0].y.shift();
            myPlot.data[1].y.shift();
        } else {
            myPlot.data[0].x.push(myPlot.data[0].x.length + 1);
            myPlot.data[1].x.push(myPlot.data[1].x.length + 1);
        }
        myPlot.data[0].y.push(y1);
        myPlot.data[1].y.push(y2);
        myPlot.redraw();
    }
}

const FileSaver = {};
const defaultFile = 'mydata.csv';
const fileHeader = 'RH %, Temperature C, Time\n'
const isOnline = typeof nw == 'undefined' || typeof process == 'undefined';
let dbgOnline = false;
if (isOnline | dbgOnline) {
    let mywriter;
    const myTextEnc = new TextEncoder();
    FileSaver.saveStart = function (filename) {
        if (!filename) filename = defaultFile;
        const mystream = streamSaver.createWriteStream(filename);
        FileSaver.saveAbort();
        mywriter = mystream.getWriter();
        FileSaver.saveData(fileHeader);
    };
    FileSaver.saveStop = function () {
        if (mywriter) {
            mywriter.close();
            mywriter = null;
        }
    };
    FileSaver.saveAbort = function () {
        if (mywriter) {
            mywriter.abort('reason');
            mywriter = null;
        }
    };
    FileSaver.saveData = function (data) {
        if (mywriter) {
            mywriter.write(myTextEnc.encode(data));
        }
    };
    window.addEventListener('unload', (event) => {
        FileSaver.saveAbort();
    });
} else {
    // node fs API: https://nodejs.org/api/fs.html
    const fs = require('fs');
    FileSaver.saveLog = false;
    FileSaver.saveFile;
    let saveStarted = undefined;

    FileSaver.saveStart = function (filename) {
        saveStarted = true;
        templateObj.$.file_input.setAttribute('nwsaveas', defaultFile);
        templateObj.$.file_input.click();
    };
    FileSaver.saveStop = function () {
        FileSaver.saveLog = false;
    };
    FileSaver.saveAbort = function () {
        FileSaver.saveLog = false;
    };
    FileSaver.saveData = function (data) {
        if (!FileSaver.saveLog) return;
        if (saveStarted) {
            saveStarted = false;
            fs.writeFileSync(FileSaver.saveFile, fileHeader, { flag: 'w' /* overwrite */ });
        };
        fs.writeFileSync(FileSaver.saveFile, data, { flag: 'a' /* append to file */ });
    };
}
/*
 *  Boilerplate code for creating computed data bindings
 */
document.addEventListener('gc-databind-ready', function () {
    const hookDecodeMethod = gc.databind.USB2ANY.prototype.decode;
    gc.databind.USB2ANY.prototype.decode = function (target, rawData) {
        const result = hookDecodeMethod.call(this, target, rawData);
        if (result === false) {
            const ERR_I2C_ADDR_NAK = -44;
            if (rawData.length > 7 && rawData[6] - 256 === ERR_I2C_ADDR_NAK && rawData[3] === 3) {
                cntNACK++;
                // console.log("cntNACK: " + cntNACK);
                if (cntNACK > 1)
                    return false;
                else
                    return true;
            }
        } else cntNACK = 0;
        return result;
    };
});

/*
 *  Boilerplate code for creating custom actions
 */
document.addEventListener('gc-nav-ready', function () {
});

/*
 *  Boilerplate code for working with components in the application gist
 */
/* setup widget event listener */
var initComplete = false;
var templateObj;
var refreshGraphRate = 0;
var hInterval = null;

const readTemperature = () => {
    // read temperature from device
    const u2a = document.querySelector('ti-model-register').getCodec();
    var HDC3Addr = templateObj.$.ti_widget_droplist_i2c_addr.selectedValue;
    let MSB;
    let LSB;
    if (chartAutoMode) {
        MSB = 0xE0;
        LSB = 0x00;
    } else {
        MSB = 0x24;
        LSB = hdc3.mode.trig.duration[chartTrigDur];
    }
    let CRC = crcHDC3([MSB, LSB]);
    if (u2a) {
        u2a.sendCommandPacket(U2A.I2C_Write, [0, HDC3Addr, 3, MSB, LSB, CRC]);
        setTimeout(() => {
            u2a.readResponse(u2a.sendCommandPacket(U2A.I2C_Read, [0, HDC3Addr, 6, 0])).then(packet => {
                const payloadLength = packet[2];
                const payload = packet.slice(8, 8 + payloadLength);
                const chk = payload[2] == crcHDC3(payload.slice(0, 2));
                let RHb = (payload[3] << 8) | payload[4];
                let TMPb = (payload[0] << 8) | payload[1];
                let RH = 100 * (RHb / (2 ** 16 - 1));
                let TMP = -45 + 175 * (TMPb / (2 ** 16 - 1));
                templateObj.$.ti_widget_markdown_numeral.markdownText = '<h1>' + RH.toFixed(2) + "%RH" + '  \n' + TMP.toFixed(2) + '°C</h1>'
                myPlotly.update(RH, TMP);
                const d = new Date();
                timestamp = d.toLocaleString().replace(',','');
                FileSaver.saveData(RH + ',' + TMP + ',' + timestamp + '\n');
                if (jdrift_en) {
                    let jslope = (TMP - pTMP) / 1;
                    if (jslope < 0.01) {
                        jdrift_en = false;
                        console.log('jdrift stop', jslope, RH);
                        HDC3Write([0x30, 0x66]); //disable heater
                        templateObj.$.ti_widget_input_rh_offset.value = -1 * (RH.toFixed(2)) / 4;
                        templateObj.$.ti_widget_label_jdriftOffset.label = "Drift Offset: " + (RH.toFixed(2)) / 4;
                    }
                }
                pTMP = TMP;
            });
        }, 2);
    }
};

const poll = () => {
    if (hInterval) {
        clearInterval(hInterval);
        hInterval = null;
    }

    if (refreshGraphRate >= 0) {
        hInterval = setInterval(() => {
            readTemperature();
        }, refreshGraphRate);
    }
};


var init = function () {
    templateObj = document.querySelector('#template_obj');
    templateObj.addEventListener('dom-change', () => {
        if (initComplete) return;

        templateObj.async(() => {
            initComplete = true;

            refreshGraphRate = templateObj.$.ti_widget_droplist_auto_read.selectedValue;
            poll();

            // droplist refresh stuff
            templateObj.$.ti_widget_droplist_auto_read.addEventListener("selected-value-changed", ({ detail }) => {
                refreshGraphRate = detail.value;
                poll();
            });

            // menu button
            templateObj.toggleMenu = function (event) {
                templateObj.$.ti_widget_vtabcontainer.isExpanded = !templateObj.$.ti_widget_vtabcontainer.isExpanded;
            };
            templateObj.$.ti_widget_icon_button_menu.addEventListener('click', templateObj.toggleMenu);
            templateObj.$.ti_widget_label_menu.addEventListener('click', templateObj.toggleMenu);

            templateObj.$.ti_widget_droplist_i2c_freq.addEventListener("selected", function (event) {
                const u2a = document.querySelector('ti-model-register').getCodec();         // get codec
                const Cmd_I2C_Set = 1;                                                     // Command to set I2C port
                const speed = templateObj.$.ti_widget_droplist_i2c_freq.selectedIndex;    // Speed based on droplist index: 0:100kHz, 1:400kHz, 2:10kHz
                u2a.sendCommandPacket(Cmd_I2C_Set, [speed, 0, 1])      // send command to set I2C, see packet definition in USB2ANY.h
                //don't implement 2:10kHz because it will cause GC hardware connection to time out
            });
            templateObj.$.ti_widget_crc_input.addEventListener("changed", function (event) {
                //const msg = _.split(templateObj.$.ti_widget_crc_input.value, " ");
                const msg = templateObj.$.ti_widget_crc_input.value.split(" ").map((s) => parseInt(s));
                templateObj.$.ti_widget_crc_output_label.label = 'CRC: 0x' + crcHDC3(msg).toString(16).toUpperCase();
            });
            //I2C General Call Reset
            templateObj.$.ti_widget_button_i2c_gcr.addEventListener("click", function (event) {
                const u2a = document.querySelector('ti-model-register').getCodec();
                const Cmd_I2C_Write = 2;
                u2a.sendCommandPacket(Cmd_I2C_Write, [0, 0, 1, 6])
            });

            const updatePlot = () => {
                const clientRect = templateObj.$.ti_widget_plot.getBoundingClientRect();
                templateObj.$.ti_widget_plot.layout.width = clientRect.width;
                templateObj.$.ti_widget_plot.layout.height = clientRect.height;
                templateObj.$.ti_widget_plot.redraw();
            };
            new ResizeObserver(updatePlot).observe(templateObj.$.ti_widget_plot);

            const setChartRange = function () {
                const range = [templateObj.$.ti_widget_input_chart_min.value, templateObj.$.ti_widget_input_chart_max.value]
                templateObj.$.ti_widget_plot.layout.yaxis.autorange = false;
                templateObj.$.ti_widget_plot.layout.yaxis2.autorange = false;
                templateObj.$.ti_widget_plot.layout.yaxis.range = range;
                templateObj.$.ti_widget_plot.layout.yaxis2.range = range;
            };
            templateObj.$.ti_widget_input_chart_min.addEventListener("change", setChartRange);
            templateObj.$.ti_widget_input_chart_max.addEventListener("change", setChartRange);

            if (!(isOnline | dbgOnline))
                // https://nwjs.readthedocs.io/en/nw13/References/File%20Dialogs/#attribute-nwsaveas
                templateObj.$.file_input.addEventListener('change', e => {
                    FileSaver.saveFile = templateObj.$.file_input.value;
                    templateObj.$.file_input.value = "";
                    FileSaver.saveLog = true;
                });
            templateObj.$.ti_widget_button_saveStart.addEventListener("click", e => FileSaver.saveStart());
            templateObj.$.ti_widget_button_saveStop.addEventListener("click", e => FileSaver.saveStop());

            const updateMMode = function () {
                const rate = templateObj.$.ti_widget_droplist_mmode.selectedValue;
                const dur = templateObj.$.ti_widget_droplist_duration.selectedIndex;
                templateObj.$.ti_widget_label_mode_msb.label = '0x' + hdc3.mode[rate].MSB.toString(16).toUpperCase();
                templateObj.$.ti_widget_label_mode_lsb.label = '0x' + hdc3.mode[rate].duration[dur].toString(16).toUpperCase();
            };
            templateObj.$.ti_widget_droplist_mmode.addEventListener("selected", updateMMode);
            templateObj.$.ti_widget_droplist_duration.addEventListener("selected", updateMMode);

            const sendMMode = function () {
                const rate = templateObj.$.ti_widget_droplist_mmode.selectedValue;
                const dur = templateObj.$.ti_widget_droplist_duration.selectedIndex;
                const MSB = hdc3.mode[rate].MSB;
                const LSB = hdc3.mode[rate].duration[dur];
                const CRC = crcHDC3([MSB, LSB]);
                chartAutoMode = (rate == 'trig') ? 0 : 1;
                chartTrigDur = dur;
                HDC3Write([MSB, LSB, CRC]);
            };
            templateObj.$.ti_widget_button_sendMMode.addEventListener("click", sendMMode);

            templateObj.$.ti_widget_button_softwareReset.addEventListener("click", e => {
                HDC3Write([0x30, 0xA2]);
            });
            templateObj.$.ti_widget_button_heaterEnable.addEventListener("click", e => {
                HDC3Write([0x30, 0x6D]);
            });
            templateObj.$.ti_widget_button_heaterDisable.addEventListener("click", e => {
                HDC3Write([0x30, 0x66]);
            });

            const updateOffsetCode = function () {
                let t = Number(templateObj.$.ti_widget_input_t_offset.value);
                let rh = Number(templateObj.$.ti_widget_input_rh_offset.value);
                let CodeMask = 0x8080;
                if (t < 0) {
                    t = Math.abs(t);
                    CodeMask &= 0xFF7F;
                }
                if (rh < 0) {
                    rh = Math.abs(rh);
                    CodeMask &= 0x7FFF;
                }
                let tCode = Math.round(t * (2 ** 16 - 1) / 175);
                let rhCode = Math.round(rh * (2 ** 16 - 1) / 100);
                let Code = ((rhCode << 1) & 0x7F00) | ((tCode >> 6) & 0x7F) | CodeMask;
                // console.log('rh:0x' + rhCode.toString(16));
                // console.log('t:0x' + tCode.toString(16));
                templateObj.$.ti_widget_input_offset_hex.value = '0x' + Code.toString(16).toUpperCase();
            };
            const updateOffsetFloat = function () {
                const payload = Number(templateObj.$.ti_widget_input_offset_hex.value);
                let RH = payload & 0x7F00;
                let TMP = payload & 0x7F;
                RH >>= 1;
                RH = convR(RH);
                TMP <<= 6;
                TMP = 175 * (TMP / (2 ** 16 - 1));
                if (!(payload & 0x8000)) {
                    RH *= -1;
                }
                if (!(payload & 0x80)) {
                    TMP *= -1;
                }
                templateObj.$.ti_widget_input_t_offset.value = TMP.toFixed(2);
                templateObj.$.ti_widget_input_rh_offset.value = RH.toFixed(2);
            };
            templateObj.$.ti_widget_input_t_offset.addEventListener("changed", updateOffsetCode);
            templateObj.$.ti_widget_input_rh_offset.addEventListener("value-changed", updateOffsetCode);
            templateObj.$.ti_widget_input_offset_hex.addEventListener("changed", updateOffsetFloat);
            templateObj.$.ti_widget_button_readOffset.addEventListener("click", function (event) {
                const MSB = 0xA0;
                const LSB = 0x04;
                HDC3Write([MSB, LSB]);
                HDC3Read(3, function (payload) {
                    const chk = payload[2] == crcHDC3(payload.slice(0, 2));
                    if (chk) {
                        templateObj.$.ti_widget_input_offset_hex.value = '0x' + (((payload[0] & 0xFF) << 8) | payload[1]).toString(16).toUpperCase();
                        updateOffsetFloat();
                    };
                });
            });
            templateObj.$.ti_widget_button_writeOffset.addEventListener("click", function (event) {
                const MSB = 0xA0;
                const LSB = 0x04;
                const payload = Number(templateObj.$.ti_widget_input_offset_hex.value);
                const pb1 = (payload >> 8) & 0xFF;
                const pb2 = payload & 0xFF;
                const CRC = crcHDC3([pb1, pb2]);
                HDC3Write([MSB, LSB, pb1, pb2, CRC]);
            });

            templateObj.$.ti_widget_button_readStatus.addEventListener("click", function (event) {
                const MSB = 0xF3;
                const LSB = 0x2D;
                HDC3Write([MSB, LSB]);
                HDC3Read(3, function (payload) {
                    const chk = payload[2] == crcHDC3(payload.slice(0, 2));
                    if (chk) {
                        templateObj.$.ti_widget_label_status.label = 'Status: 0x' + (((payload[0] & 0xFF) << 8) | payload[1]).toString(16).toUpperCase();
                        let status = [
                            /* 0 */['Last Checksum good', '**Last Checksum bad**'],
                            /* 1 */['Reserved', 'Reserved'],
                            /* 2 */['Drift correction successful', '**Drift correction error/aborted**'],
                            /* 3 */['Drift Correction is not running', '**Drift Correction is running**'],
                            /* 4 */['No reset detected', '**Reset detected**'],
                            /* 5 */['Drift Correction on Next Power Cycle Disabled', '**Drift Correction Execution on Next Power Cycle Enabled**'],
                            /* 6 */['No T Low Alert', '**T Low Alert**'],
                            /* 7 */['No T High Alert', '**T High Alert**'],
                            /* 8 */['No RH Low Alert', '**RH Low Alert**'],
                            /* 9 */['No RH High Alert', '**RH High Alert**'],
                            /* 10 */['No T Tracking Alert', '**T Tracking Alert**'],
                            /* 11 */['No RH Tracking Alert', '**RH Tracking Alert**'],
                            /* 12 */['Reserved', 'Reserved'],
                            /* 13 */['Heater Disabled', '**Heater Enabled**'],
                            /* 14 */['Reserved', 'Reserved'],
                            /* 15 */['No pending alert', '**At least one pending alert**']
                        ];
                        let c = (((payload[0] & 0xFF) << 8) | payload[1]);
                        let msg = 'Status Detail:  \n';
                        for (i = 15; i > -1; i--) {
                            if (c & (2 ** i))
                                msg += (String(i).padStart(2, '0') + '**:**1 ' + status[i][1] + '  \n');
                            else
                                msg += (String(i).padStart(2, '0') + '**:**0 ' + status[i][0] + '  \n');
                        }
                        templateObj.$.ti_widget_markdown_status.markdownText = msg;
                    };
                });
            });
            templateObj.$.ti_widget_button_clearStatus.addEventListener("click", function (event) {
                const MSB = 0x30;
                const LSB = 0x41;
                HDC3Write([MSB, LSB]);
            });

            templateObj.$.ti_widget_button_driftStart.addEventListener("click", e => {
                const MSB = 0xA0;
                const LSB = 0x01;
                HDC3Write([MSB, LSB]);
            });
            templateObj.$.ti_widget_button_driftAbort.addEventListener("click", e => {
                const MSB = 0xA0;
                const LSB = 0x07;
                HDC3Write([MSB, LSB]);
            });
            templateObj.$.ti_widget_button_readDriftOffset.addEventListener("click", function (event) {
                const MSB = 0xA0;
                const LSB = 0x02;
                HDC3Write([MSB, LSB]);
                HDC3Read(3, function (payload) {
                    const chk = payload[2] == crcHDC3(payload.slice(0, 2));
                    if (chk) {
                        templateObj.$.ti_widget_label_driftOffset.label = 'Drift Offset: 0x' + (((payload[0] & 0xFF) << 8) | payload[1]).toString(16).toUpperCase();
                    };
                });
            });
            templateObj.$.ti_widget_button_resetDriftOffset.addEventListener("click", e => {
                const MSB = 0xA0;
                const LSB = 0x03;
                HDC3Write([MSB, LSB]);
            });

            templateObj.$.ti_widget_button_jdriftStart.addEventListener("click", e => {
                const heater = templateObj.$.ti_widget_droplist_jdrift.selectedValue;
                let b1 = (heater >> 8) & 0xFF;
                let b2 = heater & 0xFF;
                let crc = crcHDC3([b1, b2]);
                HDC3Write([0xA0, 0x04, 0x00, 0x00, 0x81]); //clear any offsets
                HDC3Write([0x30, 0x6E, b1, b2, crc]);
                HDC3Write([0x30, 0x6D]);
                setTimeout(_ => { jdrift_en = true; }, 1000);
            });
            templateObj.$.ti_widget_button_jdriftAbort.addEventListener("click", e => {
                HDC3Write([0x30, 0x66]); //disable heater
            });

            templateObj.$.ti_widget_button_readNIST.addEventListener("click", e => {
                HDC3Write([0x36, 0x83]);
                HDC3Read(3, function (payload) {
                    const chk = !(crcHDC3(payload.slice(0, 3)));
                    if (chk) {
                        templateObj.$.ti_widget_markdown_nist.markdownText = 'NIST ID 5 & 4: 0x' + (((payload[0] & 0xFF) << 8) | payload[1]).toString(16).toUpperCase() + '  \n';
                    };
                });
                HDC3Write([0x36, 0x84]);
                HDC3Read(3, function (payload) {
                    const chk = !(crcHDC3(payload.slice(0, 3)));
                    if (chk) {
                        templateObj.$.ti_widget_markdown_nist.markdownText += 'NIST ID 3 & 2: 0x' + (((payload[0] & 0xFF) << 8) | payload[1]).toString(16).toUpperCase() + '  \n';
                    };
                });
                HDC3Write([0x36, 0x85]);
                HDC3Read(3, function (payload) {
                    const chk = !(crcHDC3(payload.slice(0, 3)));
                    if (chk) {
                        templateObj.$.ti_widget_markdown_nist.markdownText += 'NIST ID 1 & 0: 0x' + (((payload[0] & 0xFF) << 8) | payload[1]).toString(16).toUpperCase();
                    };
                });
            });

            const updateLimitCode = function () {
                let t = Number(templateObj.$.ti_widget_input_t_limit.value);
                let rh = Number(templateObj.$.ti_widget_input_rh_limit.value);
                let tCode = Math.round((t + 45) * (2 ** 16 - 1) / 175);
                let rhCode = Math.round(rh * (2 ** 16 - 1) / 100);
                let Code = (rhCode & 0xFE00) | ((tCode >> 7) & 0x1FF);
                // console.log('rh:0x' + rhCode.toString(16));
                // console.log('t:0x' + tCode.toString(16));
                templateObj.$.ti_widget_input_limit_hex.value = '0x' + Code.toString(16).toUpperCase();
            };
            const updateLimitFloat = function () {
                let payload = Number(templateObj.$.ti_widget_input_limit_hex.value);
                let RH = payload & 0xFE00;
                let TMP = payload & 0x1FF;
                // RH <<= 9;
                RH = 100 * (RH / (2 ** 16 - 1));
                TMP <<= 7;
                TMP = -45 + 175 * (TMP / (2 ** 16 - 1));
                templateObj.$.ti_widget_input_t_limit.value = TMP.toFixed(2);
                templateObj.$.ti_widget_input_rh_limit.value = RH.toFixed(2);
            };
            templateObj.$.ti_widget_input_t_limit.addEventListener("changed", updateLimitCode);
            templateObj.$.ti_widget_input_rh_limit.addEventListener("changed", updateLimitCode);
            templateObj.$.ti_widget_input_limit_hex.addEventListener("changed", updateLimitFloat);
            templateObj.$.ti_widget_button_readAlert.addEventListener("click", function (event) {
                const val = templateObj.$.ti_widget_droplist_alert.selectedValue;
                const MSB = hdc3.vrfyAlert.MSB;
                const LSB = hdc3.vrfyAlert[val];
                const CRC = crcHDC3([MSB, LSB]);
                HDC3Write([MSB, LSB, CRC]);
                HDC3Read(3, function (payload) {
                    const chk = payload[2] == crcHDC3(payload.slice(0, 2));
                    if (chk) {
                        templateObj.$.ti_widget_input_limit_hex.value = '0x' + (((payload[0] & 0xFF) << 8) | payload[1]).toString(16).toUpperCase();
                        updateLimitFloat();
                    }
                });
            });
            templateObj.$.ti_widget_button_writeAlert.addEventListener("click", function (event) {
                const val = templateObj.$.ti_widget_droplist_alert.selectedValue;
                const MSB = hdc3.cfgAlert.MSB;
                const LSB = hdc3.cfgAlert[val];
                const payload = Number(templateObj.$.ti_widget_input_limit_hex.value);
                const pb1 = (payload >> 8) & 0xFF;
                const pb2 = payload & 0xFF;
                const CRC = crcHDC3([pb1, pb2]);
                HDC3Write([MSB, LSB, pb1, pb2, CRC]);
            });
        }, 1);

    });
};

templateObj = document.querySelector('#template_obj');
if (templateObj) {
    init();
} else {
    document.addEventListener('DOMContentLoaded', init);
}